@@ -1,5 +1,4 @@
 #include <fstream>
-#include <set>
 #include <string>
 
 #include <boost/algorithm/string/erase.hpp>
@@ -7,9 +6,7 @@
 #include <boost/algorithm/string/replace.hpp> // replace_all_copy
 
 #include <lib/vcsn/algos/fwd.hh>
-#include <lib/vcsn/algos/registry.hh>
 #include <vcsn/algos/edit-automaton.hh>
-#include <vcsn/dyn/registries.hh>
 #include <vcsn/dyn/algos.hh>
 #include <vcsn/dyn/automaton.hh>
 #include <vcsn/misc/getargs.hh>
@@ -27,8 +24,9 @@ namespace vcsn
       std::string
       read_here_doc(std::istream& is)
       {
-        static std::regex re("cat >\\$medir/([a-z]+)\\.[a-z]* <<\\\\EOFSM",
-                             std::regex::extended);
+        static const auto re
+          = std::regex("cat >\\$medir/([a-z]+)\\.[a-z]* <<\\\\EOFSM",
+                       std::regex::extended);
         std::string line;
         std::smatch res;
         while (is.good())
@@ -100,7 +98,7 @@ namespace vcsn
     }
 
     automaton
-    read_efsm(std::istream& is)
+    read_efsm(std::istream& is, const location&)
     {
       std::string file = "file.efsm";
       using string_t = symbol;
