@@ -1,4 +1,4 @@
-#include <vcsn/dyn/translate.hh>
+#include <lib/vcsn/dyn/translate.hh>
 
 #include <chrono>
 #include <fstream>
@@ -8,22 +8,34 @@
 #include <string>
 #include <unistd.h> // getpid
 
+#include <boost/algorithm/string/case_conv.hpp>
 #include <boost/filesystem.hpp>
 
-#include <ltdl.h>
+#include <lib/vcsn/dyn/context-parser.hh>
+#include <lib/vcsn/dyn/context-printer.hh>
+#include <lib/vcsn/dyn/type-ast.hh>
+#include <lib/vcsn/misc/xltdl.hh>
 
-#include <vcsn/dyn/context-parser.hh>
-#include <vcsn/dyn/context-printer.hh>
-
-#include <vcsn/config.hh>
 #include <vcsn/dyn/context.hh>
+#include <vcsn/misc/configuration.hh>
 #include <vcsn/misc/escape.hh>
 #include <vcsn/misc/indent.hh>
-#include <vcsn/misc/raise.hh>
 #include <vcsn/misc/regex.hh>
 #include <vcsn/misc/signature.hh>
 #include <vcsn/misc/stream.hh>
 
+namespace
+{
+  /// Level of verbosity.
+  int verbose = []{
+    auto res = 0;
+    auto cp = getenv("VCSN_VERBOSE");
+    std::istringstream is{cp ? cp : "0"};
+    is >> res;
+    return res;
+  }();
+}
+
 namespace vcsn
 {
   namespace dyn
@@ -36,16 +48,20 @@ namespace vcsn
 
     namespace
     {
-      /// \a getenv(var) if defined, otherwise \a val.
-      std::string
-      xgetenv(const std::string& var, const std::string& val = "")
-      {
-        const char* cp = getenv(var.c_str());
-        return cp ? cp : val;
-      }
-
 #define XGETENV(Name) xgetenv(#Name, Name)
 
+      /// Get a var from the config, unless an envvar override it.
+      /// For instance `configuration.cxx` and `$VCSN_CXX`.
+      std::string
+      config(const std::string& var)
+      {
+        auto envvar = "VCSN_" + boost::algorithm::to_upper_copy(var);
+        if (auto cp = getenv(envvar.c_str()))
+          return cp;
+        else
+          return get_config()["configuration"][var].str();
+      }
+
       /// Expand initial "~" in res.
       // http://stackoverflow.com/questions/4891006.
       std::string expand_tilda(std::string res)
@@ -84,7 +100,7 @@ namespace vcsn
       struct translation
       {
         translation()
-          : parser_(is), printer_(os)
+          : printer_(os)
         {}
 
         /// Generate the code to compile in file \a base ".cc".
@@ -94,6 +110,9 @@ namespace vcsn
         /// BASE.so.  Rather, we use BASE.PID.cc -> BASE.cc ->
         /// BASE.PID.o, other ccache will always fail to find
         /// "BASE.PID.cc" in its cache.
+        ///
+        /// Do it only if needed: keep the mtime old if the file
+        /// contents is the same anyway.
         void print(const std::string& base)
         {
           ensure_parent_directory(base);
@@ -104,26 +123,23 @@ namespace vcsn
             std::ofstream o{tmp + ".cc"};
             printer_.print(o);
           }
-          boost::filesystem::rename(tmp + ".cc", base + ".cc");
+          if (equal_files(tmp + ".cc", base + ".cc"))
+            boost::filesystem::remove(tmp + ".cc");
+          else
+            boost::filesystem::rename(tmp + ".cc", base + ".cc");
         }
 
         /// Generate C++ syntax for context \a ctx (which might use
         /// our syntax: 'lal_char(ab), z').
         void print_context(const std::string& ctx)
         {
-          is.clear();
-          is.str(ctx);
-          auto ast = parser_.parse_context();
-          ast->accept(printer_);
+          ast::parse_context(ctx)->accept(printer_);
         }
 
         /// Generate C++ syntax for type \a type.
         void print_type(const std::string& type)
         {
-          is.clear();
-          is.str(type);
-          auto ast = parser_.parse();
-          ast->accept(printer_);
+          ast::parse_type(type)->accept(printer_);
         }
 
         /// Parse compiler errors and throw.
@@ -160,7 +176,7 @@ namespace vcsn
             if (std::regex_search(line, smatch, r1)
                 || std::regex_search(line, smatch, r2))
               assertions += std::string(smatch[1]) + '\n';
-          if (getenv("VCSN_VERBOSE"))
+          if (verbose)
             {
               cmd += "\n  compiler error messages:\n";
               cmd += get_file_contents(err);
@@ -176,7 +192,7 @@ namespace vcsn
         {
           auto err = tmp + ".err";
           if (getenv("VCSN_DEBUG"))
-            std::cerr << "run: " << cmd << std::endl;
+            std::cerr << "run: " << cmd << '\n';
           if (system((cmd + " 2>'" + err + "'").c_str()))
             throw_compiler_errors(cmd, err);
           else
@@ -201,10 +217,10 @@ namespace vcsn
           // We try to read the error message via a regexp below.  So
           // avoid translation (we once had "erreur" instead of "error").
           auto cmd = (std::string{"LC_ALL=C"}
-                      + " " + XGETENV(VCSN_CCACHE)
-                      + " " + XGETENV(VCSN_CXX)
-                      + " " + XGETENV(VCSN_CXXFLAGS)
-                      + " " + XGETENV(VCSN_CPPFLAGS)
+                      + " " + config("ccache")
+                      + " " + config("cxx")
+                      + " " + config("cxxflags")
+                      + " " + config("cppflags")
                       + " -fPIC '" + base + ".cc' -c"
                       + " -o '" + tmp + ".o'");
           cxx(cmd, tmp);
@@ -217,9 +233,9 @@ namespace vcsn
         {
           auto tmp = tmpname(base);
           auto cmd = (std::string{"LC_ALL=C"}
-                      + " " + XGETENV(VCSN_CXX)
-                      + " " + XGETENV(VCSN_CXXFLAGS)
-                      + " " + XGETENV(VCSN_LDFLAGS)
+                      + " " + config("cxx")
+                      + " " + config("cxxflags")
+                      + " " + config("ldflags")
                       + " -fPIC -lvcsn '" + tmp + ".o' -shared"
                       + " -o '" + tmp + ".so'"
                       + printer_.linkflags());
@@ -239,7 +255,7 @@ namespace vcsn
         /// inserted to avoid file name limits.
         std::string split(const std::string& s) const
         {
-          std::string res;
+          auto res = std::string{};
           const size_t size = 150;
           for (unsigned i = 0; i < s.length(); i += size)
             {
@@ -280,10 +296,13 @@ namespace vcsn
               }
             else
               {
+                auto cmd
+                  = xgetenv("VCSN_COMPILE",
+                            xgetenv("VCSN", "vcsn") + " compile");
                 auto linkflags = printer_.linkflags();
                 if (!linkflags.empty())
-                  linkflags = " --extra-ldflags='" + linkflags + "'";
-                cxx("vcsn compile -shared '" + base + ".cc'" + linkflags,
+                  linkflags = " LDFLAGS+='" + linkflags + "'";
+                cxx(cmd + " -shared" + linkflags + " '" + base + ".cc'",
                     tmp);
               }
             auto d
@@ -300,22 +319,18 @@ namespace vcsn
                   std::cerr << d.count() << "ms: " << base << '\n';
               }
           }
-          static bool first = true;
-          if (first)
-            {
-              lt_dlinit();
-              first = false;
-            }
-          lt_dlhandle lib = lt_dlopen((base + ".so").c_str());
-          VCSN_REQUIRE(lib, "cannot load lib: ", base, ".so: ", lt_dlerror());
+          vcsn::detail::xlt_advise()
+            .global(true)
+            .ext()
+            .verbose(1 < verbose)
+            .open(base + ".so");
         }
 
         /// Compile, and load, a DSO with instantiations for \a ctx.
         void compile(const std::string& ctx)
         {
           printer_.header("vcsn/ctx/instantiate.hh");
-          std::string base =
-            plugindir() + "contexts/" + split(ctx);
+          auto base = plugindir() + "contexts/" + split(ctx);
           os << "using ctx_t =" << incendl;
           print_context(ctx);
           os << ';' << decendl
@@ -334,7 +349,7 @@ namespace vcsn
         void compile(const std::set<std::pair<std::string, signature>>& algos)
         {
           printer_.header("vcsn/misc/attributes.hh"); // ATTRIBUTE_USED
-          printer_.header("vcsn/misc/name.hh"); // ssignature
+          printer_.header("vcsn/dyn/name.hh"); // ssignature
           printer_.header("vcsn/dyn/registries.hh");
           for (const auto& algo: algos)
             printer_.header_algo(algo.first);
@@ -359,28 +374,29 @@ namespace vcsn
                 }
               os <<
                 "\n"
-                "static bool " << algo.first << " ATTRIBUTE_USED =" << incendl
-                 << "vcsn::dyn::detail::" << algo.first << "_register(" << incendl
-                 << "vcsn::ssignature<" << types << ">()," << iendl
-                 << "vcsn::dyn::detail::" << algo.first << "<" << types << ">" << decendl
+                "static bool vcsn_" << algo.first << " ATTRIBUTE_USED ="
+                 << incendl
+                 << "vcsn::dyn::detail::" << algo.first << "_register("
+                 << incendl
+                 << "vcsn::ssignature<" << types << ">(),"
+                 << iendl
+                 << "vcsn::dyn::detail::" << algo.first << "<" << types << ">"
+                 << decendl
                  << ");" << decendl;
             }
 
           // The first algo is the once that gives its name to the
           // file to compile.
-          std::string base = (plugindir()
-                              + "algos/"
-                              + begin(algos)->first + "/"
-                              + split(begin(algos)->second.to_string()));
+          auto base = (plugindir()
+                       + "algos/"
+                       + begin(algos)->first + "/"
+                       + split(begin(algos)->second.to_string()));
           print(base);
           jit(base);
         }
 
-        /// The input stream: the specification to translate.
-        std::istringstream is;
         /// The output stream: the corresponding C++ snippet to compile.
         std::ostringstream os;
-        ast::context_parser parser_;
         ast::context_printer printer_;
       };
     } // namespace detail
