@@ -24,21 +24,22 @@ contexts = {
 }
 
 def context_update():
-  global context, ctx
+  global context, ctx # pylint: disable=global-statement
   if context in contexts:
     context = contexts[context]
   ctx = vcsn.context(context)
   print("# context: {} ({})".format(context, ctx))
 
 
-def expr(re):
+def expr(e):
   '''Parse.  If it fails, prepend "! " to the error
   message and return it as result.  Strip the "try -h" line.'''
   try:
-    print(re, identities)
-    return ctx.expression(re, identities)
+    print(e, identities)
+    return ctx.expression(e, identities)
   except RuntimeError:
-    return "! " + str(sys.exc_info()[1])
+    err = re.sub('^', '! ', str(sys.exc_info()[1]), flags=re.M)
+    return r'\n'.join(err.splitlines())
 
 def pp(re):
   '''Parse and pretty-print.  If it fails, prepend "! " to the error
@@ -48,7 +49,7 @@ def pp(re):
 def check_rat_exp(fname):
   file = open(fname, 'r', encoding='utf-8')
   lineno = 0
-  global context, identities
+  global context, identities # pylint: disable=global-statement
   for line in file:
     lineno += 1
     loc = fname + ':' + str(lineno)
@@ -98,7 +99,7 @@ def check_rat_exp(fname):
 
     # == tests that boths are equivalent.
     # => check the actual result.
-    m = re.match('(.*\S)\s*(=>|==)\s*(.*)$', line)
+    m = re.match(r'(.*\S)\s*(=>|==)\s*(.*)$', line)
     if m is not None:
       l = m.group(1)
       op = m.group(2)
@@ -117,7 +118,7 @@ def check_rat_exp(fname):
       continue
 
     # !: Look for syntax errors.
-    m = re.match('(.*\S)\s+(!.*)$', line)
+    m = re.match(r'(.*\S)\s+(!.*)$', line)
     if m is not None:
       l = m.group(1)
       err = m.group(2)
