@@ -34,6 +34,15 @@ CHECK(a.is_ambiguous())
 CHECK_EQ("a", a.ambiguous_word())
 CHECK(not a.is_deterministic())
 
+# Check the ambiguous word for a not so trivial case.
+CHECK_EQ('introABCDEFoutro',
+         vcsn.B.expression('intro (ABC + DEF + AB + CD + E + F)* outro').standard()
+         .ambiguous_word())
+# Another type of ambiguity.
+CHECK_EQ('introAoutro',
+         vcsn.B.expression('intro A{+}{+} outro').thompson()
+         .ambiguous_word())
+
 # Test ambiguous word on automata with two ambiguous parts. Hence,
 # test monomial_path with states different from pre and post.
 b = vcsn.automaton('''
@@ -86,6 +95,29 @@ CHECK(not a.is_ambiguous())
 XFAIL(lambda: a.ambiguous_word(), "automaton is unambiguous")
 CHECK(a.is_deterministic())
 
+# Retrieve shortest word, not lightest word.
+# When using lightest to retrieve the shortest ambiguous word, ambiguous_word
+# would return abb instead of ab.
+a = vcsn.automaton('''
+digraph
+{
+  vcsn_context="lal_char(ab), nmin"
+  I -> 1
+  1 -> 2 [label = "a"]
+  1 -> 3 [label = "a"]
+  2 -> 2 [label = "b"]
+  2 -> 7 [label = "<4>b"]
+  3 -> 4 [label = "b"]
+  3 -> 5 [label = "<4>b"]
+  4 -> 6 [label = "b"]
+  5 -> F
+  6 -> F
+  7 -> F
+}
+''')
+CHECK(a.is_ambiguous())
+CHECK_EQ("ab", a.ambiguous_word())
+CHECK(not a.is_deterministic())
 
 ## -------------------- ##
 ## is_cycle_ambiguous.  ##
@@ -138,20 +170,17 @@ digraph
 }''')
 CHECK(aut3.is_cycle_ambiguous())
 
+ctx = vcsn.context("lal_char(abc), b")
+
 r1 = "((abc)*){5}abc" + format(aut1.expression())
-aut4 = vcsn.context("lal_char(abc), b").expression(r1).derived_term()
+aut4 = ctx.expression(r1).derived_term()
 CHECK(aut4.is_cycle_ambiguous())
 
 r2 = "((abc)*){5}abc" + format(aut2.expression())
-aut5 = vcsn.context("lal_char(abc), b").expression(r2).derived_term()
+aut5 = ctx.expression(r2).derived_term()
 CHECK(not aut5.is_cycle_ambiguous())
 
-
-CHECK(vcsn.context("lal_char(abc), b").
-         ladybird(5).is_cycle_ambiguous())
-CHECK(not vcsn.context("lal_char(abc), b").
-         de_bruijn(5).is_cycle_ambiguous())
-CHECK(vcsn.context("lal_char(abc), b").
-         ladybird(20).is_cycle_ambiguous())
-CHECK(not vcsn.context("lal_char(abc), b").
-        de_bruijn(20).is_cycle_ambiguous())
+CHECK(ctx.ladybird(5).is_cycle_ambiguous())
+CHECK(not ctx.de_bruijn(5).is_cycle_ambiguous())
+CHECK(ctx.ladybird(20).is_cycle_ambiguous())
+CHECK(not ctx.de_bruijn(20).is_cycle_ambiguous())
