@@ -5,16 +5,14 @@ from test import *
 
 z = vcsn.context('lal_char(01), z')
 
-# FIXME: check that only the accessible part is kept.  Start from
-# binary.gv, modified with an inaccessible state.
 binary = load('lal_char_z/binary.gv')
 
 # power 0.
-CHECK_EQ(vcsn.automaton(filename = medir + '/binary^0.gv'),
+CHECK_EQ(meaut('binary^0.gv'),
          binary & 0)
 
 # power 1.
-CHECK_EQ(vcsn.automaton(filename = medir + '/binary^1.gv'),
+CHECK_EQ(meaut('binary^1.gv'),
          binary & 1)
 
 # power 4.
@@ -35,3 +33,10 @@ CHECK_EQ(z.weight('32'), binary5('10'))
 # Power 7.
 binary7 = binary & 7
 CHECK_EQ(z.weight('128'), binary7('10'))
+
+# Check that only the accessible part is kept using binary.gv,
+# modified with an inaccessible part.
+inacc = binary + vcsn.automaton('''context = lal, z
+0 -> 1 0''')
+for i in [0, 1, 4, 7]:
+    CHECK_EQ(binary & i, (inacc & i).__value__())
